package com.mysql.cluster.ndbj;

import java.math.BigInteger;

/**
*
* This class represents the NDB kernel; it is the primary class of the NDB API.
* <br>An <code>Ndb</code> object is used to create transaction objects for use on a specific database.
* They are constructed using: a <code>NdbClusterConnection</code> object; the name of the database schema to be accessed;
* and the maximum number of transactions that will be created using the <code>Ndb</code> object.
 * <br>You must call the close() method on the <code>Ndb</code> object when you have finished using
* it to free resources.
* <br><code>Ndb</code> objects are not thread-safe. Howver, it is possible to implement a multi-threaded application
* using a single, non-shared <code>Ndb</code> object per thread or by providing synchronised access to a shared <code>Ndb</code> object.
* <br>A single application can support a maximum of 128 <code>Ndb</code> objects.
*
* <br>Sample Code:<br>
* <code>
*      NdbClusterConnection conn=null;<br>
*      Ndb ndb=null;<br>
*      try {<br>
*          connRef = NdbFactory.createNdbClusterConnection("ndb-connectstring");<br>
*          ndb = conn.createNdb("database",12); // Max num of concurrent transactions is set to 12 <br>
*      }<br>
*      catch (NdbApiException e) {<br>
*          // Handle Error
*      }<br>
*       finally {<br>
*           // You must close the ndb object to free up native resources.<br>
*          if  (ndb != null)<br>
*              ndb.close();<br>
*          if (conn != null)<br>
*              conn.close();<br>
*       }<br>
* }<br>
*</code>
* @see      NdbClusterConnection
* @see      NdbTransaction
*/
public interface Ndb
{

    /**
     * init is deprecated. Please specify max transaction in the NdbClusterConnection.createNdb()
     * method instead.
     * @param maxNumberOfTransactions This parameter specifies the maximum number of parallel
     * NdbTransaction objects that can be handled by this instance of Ndb.
     * The maximum permitted value for maxNoOfTransactions is 1024;
     * if not specified Note each scan or index operation uses an extra NdbTransaction object.
     * @throws NdbApiException if an error occured in the native library
     * @throws IllegalStateException if the close method has already been called on the object.
     */
	@Deprecated
    public abstract int init(int maxNumberOfTransactions) throws NdbApiException;

    /**
     * Creates a NdbTransaction object that can be used to create
     * an NdbOperation object for reading, inserting and updating
     * rows in a schema.
     * When no schema is specfied, the transaction object will be started
     * on a random node in the cluster.
     * @return an NdbTransaction object
     * @throws NdbApiException if an error occured in the native library
     * @throws IllegalStateException if the close method has already been called on the object.
     * @see NdbTransaction
     * @see NdbOperation
     */
    public abstract NdbTransaction startTransaction() throws NdbApiException;

    /**
     * Creates a NdbTransaction object that can be used to create
     * an NdbOperation object for reading, inserting and updating
     * rows in a schema.
     * The schema name and distribution key provide a hint as to where to start the transaction.
     * The transaction should start on the node where the data is located, removing a potential
     * network hop from the transaction coordinator to the data.
     *
     * @param tableName schema name used for deciding which node to run the Transaction Coordinator on
     * @param distkey partition key corresponding to table
     * @return an NdbTransaction object
     * @throws NdbApiException if an error occured in the native library
     * @throws IllegalStateException if the close method has already been called on the object.
     * @see NdbTransaction
     */
    public abstract NdbTransaction startTransaction(String tableName, String distkey) throws NdbApiException;

    /**
    *
    * Creates a NdbTransaction object that can be used to create
    * an NdbOperation object for reading, inserting and updating
    * rows in a schema.
    * The schema name and distribution key provide a hint as to where to start the transaction.
    * The transaction should start on the node where the data is located, removing a potential
    * network hop from the transaction coordinator to the data.
    *
    * @param table Table object used for deciding which node to run the Transaction Coordinator on
    * @param distkey partition key corresponding to table
    * @return an NdbTransaction object
    * @throws NdbApiException if an error occured in the native library
    * @throws IllegalStateException if the close method has already been called on the object.
    * @see NdbTransaction
    */
	public abstract NdbTransaction startTransaction(NdbTable table, String keyData) throws NdbApiException;
    /**
    *
    * Creates a NdbTransaction object that can be used to create
    * an NdbOperation object for reading, inserting and updating
    * rows in a schema.
    * The schema name and distribution key provide a hint as to where to start the transaction.
    * The transaction should start on the node where the data is located, removing a potential
    * network hop from the transaction coordinator to the data.
    *
    * @param tableName schema name used for deciding which node to run the Transaction Coordinator on
    * @param distkey partition key corresponding to table
    * @return an NdbTransaction object
    * @throws NdbApiException if an error occured in the native library
    * @throws IllegalStateException if the close method has already been called on the object.
    * @see NdbTransaction
    */
    public abstract NdbTransaction startTransaction(String tableName, int distkey) throws NdbApiException;


    /**
    *
    * Creates a NdbTransaction object that can be used to create
    * an NdbOperation object for reading, inserting and updating
    * rows in a schema.
    * The schema name and distribution key provide a hint as to where to start the transaction.
    * The transaction should start on the node where the data is located, removing a potential
    * network hop from the transaction coordinator to the data.
    *
    * @param table Table object used for deciding which node to run the Transaction Coordinator on
    * @param distkey partition key corresponding to table
    * @return an NdbTransaction object
    * @throws NdbApiException if an error occured in the native library
    * @throws IllegalStateException if the close method has already been called on the object.
    * @see NdbTransaction
    */
	public abstract NdbTransaction startTransaction(NdbTable table, int distkey) throws NdbApiException;
    
	/**
     * Call this method when the Ndb object is no longer required
     * to free up memory allocated in the underlying native library.
     * <br>Often, this is done by calling close in a finally clause after the scope of where the
     * ndb object is used.
     */
    public abstract void close();
    
    /**
     * Call this to fetch an auto increment value from the named table. 
     * @param aTableName The table name from which to fetch
     * @param cacheSize The number of values to prefetch and cache saving subsequent round trips.
     * @return The auto increment value
     * @throws NdbApiException
     */
    public abstract BigInteger getAutoIncrementValue(String aTableName, long cacheSize) throws NdbApiException;

    /**
     * Call this to fetch an auto increment value from the named table. 
     * @param myTable The table object from which to fetch
     * @param cacheSize The number of values to prefetch and cache saving subsequent round trips.
     * @return The auto increment value
     * @throws NdbApiException
     */
    public abstract BigInteger getAutoIncrementValue(NdbTable myTable, long cacheSize) throws NdbApiException;
    
	public abstract NdbDictionary getDictionary() throws NdbApiException;

	public abstract int dropEventOperation(NdbEventOperationImpl eventOp) throws NdbApiException;
	
	public abstract NdbEventOperation createEventOperation(String eventName) throws NdbApiException;

	public abstract NdbEventOperationImpl nextEvent();

	public int pollEvents(int aMillisecondNumber) throws NdbApiException;

	public abstract int sendPollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup, int forceSend) throws NdbApiException;

	public abstract int sendPollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup) throws NdbApiException;
	
    public abstract int sendPollNdb(int aMillisecondNumber) throws NdbApiException;

	public void sendPreparedTransactions(int forceSend);

	public void sendPreparedTransactions();

	public int pollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup) throws NdbApiException;

	public int pollNdb(int aMillisecondNumber) throws NdbApiException;

	public int pollNdb() throws NdbApiException;

	public NdbTransactionImpl startTransaction(String aTableName, short keyData) throws NdbApiException
;
	public NdbTransactionImpl startTransaction(String aTableName, long keyData) throws NdbApiException;

	public NdbTransactionImpl startTransaction(NdbTable table, short keyData) throws NdbApiException;

	public NdbTransactionImpl startTransaction(NdbTable table, long keyData) throws NdbApiException;

}
