package com.mysql.cluster.ndbj;

import java.sql.Timestamp;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;

/**
 * The NdbBaseOperation interface defines get<> operations supported by
 * the four different types of Operations:
 * <ul>
 * <li> Primary key operations (NdbOperation interface )
 * <li> Scan operations (NdbScanOperation interface) 
 * <li> Index scan operations (NdbIndexScanOperation)
 * <li> Unique Index (unique key) Operations (NdbIndexOperation interface)
 * </ul> 
 * <p>The get<> operations can be understood in SQL language as being "SELECT"-style
 * operations. They indicate that this operation would like to SELECT (or get)
 * a particular column using the operation. 
 * <p>
 * The getValue method is used to select columns for all the supported primitive types (CHAR, VARCHAR, TEXT, VARBINARY, TINYBLOB, BLOB,
 * MEDIUMBLOB, SMALLINT, INTEGER, BIGINT, FLOAT,DOUBLE, TIMESTAMP).
 * <br>For Blob columns, the getBlob method is used to select a column, as blobs are handled
 * differently by NDB.
 * <p>The resultData method is used to return the NdbResultSet (similar to <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/sql/ResultSet.html">ResultSet</a> from JDBC).
 * <p>Examples:
 * For a primary key operation (NdbOperation), the following SQL: 
 * <br><code>
 *   SELECT myInt, myString, myBlob FROM my_table where pkValue='X';
 * </code>
 * <p>
 * becomes:
 * <br>
 * <code>
 *   <br>NdbOperation op = trans.getNdbOperation("my_table");
 *   <br>op.readTuple(NdbOperation.LockMode.LM_CommittedRead);
 *   <br>op.equals("pkValue","X");
 *   <br>op.getValue("myInt");
 *   <br>op.getValue("myString");
 *   <br>op.getBlob("myBlob");
 *   <br>NdbResultSet rs = op.resultData();
 * </code>
 * <p>
 * For a scan operation (NdbScanOperation), the following SQL: 
 * <br><code>
 *   SELECT myValue,myBlob FROM my_table;
 * </code>
 * <p>
 * becomes:
 * <br>
 * <code>
 *   <br>NdbScanOperation op = trans.getNdbScanOperation("my_table");
 *   <br>op.readTuples(NdbOperation.LockMode.LM_CommittedRead,0,0);
 *   <br>op.getValue("myValue");
 *   <br>op.getValue("myBlob");
 *   <br>NdbResultSet rs = op.resultData();
 * </code>
 * <p>
 * For an index scan operation (NdbIndexScanOperation), the following SQL: 
 * <br><code>
 *   SELECT myValue,myBlob FROM my_table where ;
 * </code>
 * <p>
 * becomes:
 * <br>
 * <code>
 *   <br>NdbIndexScanOperation op = trans.getNdbScanOperation("my_table","my_index_name");
 *   <br>op.readTuples(NdbOperation.LockMode.LM_CommittedRead,0,0);
 *   <br>op.getValue("myValue");
 *   <br>op.getValue("myBlob");
 *   <br>NdbResultSet rs = op.resultData();
 * </code>
 * <p>
 * For an operation on a unique index (NdbIndexOperation), the following SQL: 
 * <br><code>
 *   SELECT timeOfDay,myUniqueFloat FROM my_table where uniqueKeyCol=200;
 * </code>
 * <p>
 * becomes:
 * <br>
 * <code>
 *   <br>NdbIndexOperation op = trans.getNdbScanOperation("my_table");
 *   <br>op.readTuple(NdbOperation.LockMode.LM_CommittedRead);
 *   <br>op.equals("uniqueKeyCol",200);
 *   <br>op.getValue("timeOfDay");
 *   <br>op.getValue("myUniqueFloat");
 *   <br>NdbResultSet rs = op.resultData();
 * </code>
 * <p>
 * <table>
 * <tr><td><strong>MySQL Type</strong></td><td><strong>Java Type</strong>
 * </td></tr><tr><td>CHAR      </td><td>String
 * </td></tr><tr><td>VARCHAR   </td><td>String
 * </td></tr><tr><td>TEXT      </td><td>String
 * </td></tr><tr><td>VARBINARY </td><td>byte[]
 * </td></tr><tr><td>TINYBLOB  </td><td>byte[]
 * </td></tr><tr><td>BLOB      </td><td>byte[]
 * </td></tr><tr><td>MEDIUMBLOB</td><td>byte[]
 * </td></tr><tr><td>SMALLINT  </td><td>short
 * </td></tr><tr><td>INTEGER   </td><td>int
 * </td></tr><tr><td>BIGINT    </td><td>long
 * </td></tr><tr><td>FLOAT     </td><td>float
 * </td></tr><tr><td>DOUBLE    </td><td>double
 * </td></tr><tr><td>TIMESTAMP </td><td>java.sql.Timestamp
 * </td></tr><tr><td>TIMESTAMP </td><td>java.sql.Date
 * </td></tr></table>
 * @see NdbOperation
 * @see NdbScanOpertion
 * @see NdbIndexScanOperation
 * @see NdbIndexOperation
 * @see NdbBlob
 * @see NdbResultSet
 * @author jdowling
 */
public interface NdbBaseOperation extends NdbOperationResults {

	
	/**
     * Blobs are handled differently to other attributes in mysql cluster.
     * The getBlob operation specifies that the operation will select a blob, identified
     * by a column id.
     * <code>
     * <br>NdbOperation op = trans.getNdbOperation("someTable");
     * <br>op.equals("pkValue","X");
     * <br>op.getBlob(3); //column with offset '3' (4th column in the table)
     * <br>NdbResultSet rs = op.resultData();
     * <br>NdbBlob b = rs.getBlob(3);
     * <br>trans.execute(NdbTransaction.ExecType.NoCommit, NdbTransaction.AbortOption.AbortOnError, 1);
     * </code>
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
     * @throws NdbApiException if there was a problem in the cluster when selecting this column.
     * @throws NdbApiRuntimeException If a bad columnName is entered
	 *
	public abstract void getBlob(long columnId) throws NdbApiException;

	/**
	 * Blobs are handled differently to other attributes in mysql cluster.
	 * The getBlob operation specifies that the operation will select a blob for reading,
	 * identified by a columnName.
	 * <code>
	 * <br>NdbOperation op = trans.getNdbOperation("someTable");
     * <br>op.equals("pkValue","X");
	 * <br>op.getBlob("id"); //column with name 'id'
	 * <br>NdbResultSet rs = op.resultData();
	 * <br>NdbBlob b = rs.getBlob("id");
	 * <br>trans.execute(NdbTransaction.ExecType.NoCommit, NdbTransaction.AbortOption.AbortOnError, 1);
	 * </code>
	 *
	 * @param columnName Name of the Column
	 * @throws NdbApiException if there was a problem in the cluster when selecting this column.
     * @throws NdbApiRuntimeException If a bad columnName is entered
	 *
	public abstract void getBlob(String columnName) throws NdbApiException;

    /**
     * This method reads a number of bytes from a blob and
     * sets the current read/write position to the point following the end of the data which was read.
     * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema) 
     * @param len amount of bytes to read in the blob
     * @throws NdbApiException if there was a problem in the cluster when selecting this column.
     * @throws NdbApiRuntimeException If a bad columnName is entered
     *
    public abstract void getBlob(long columnId, int len) throws NdbApiException;
    
	/**
	 * This method reads a number of bytes from a blob and
     * sets the current read/write position to the point following the end of the data which was read.
     * @param columnName Name of the Column
     * @len amount of bytes in the Blob to read
     * @throws NdbApiException if there was a problem in the cluster when selecting this column.
     * @throws NdbApiRuntimeException If a bad columnName is entered
	 *
	public abstract void getBlob(String columnName, int len)
			throws NdbApiException;
*/
	/**
	 * Creates a blob handle NdbBlob.
	 * <br>A second call with same argument returns the previously created handle.
	 * The handle is linked to the operation and is maintained automatically.
	 *
	 * @param  columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @return NdbBlob object
	 * @throws NdbApiException if there was a problem in the cluster when selecting this column.
	 * @note getting a blob handle on a non-existing column yields a seg fault,
	 *       see bug report http://bugs.mysql.com/bug.php?id=21036
	 *
	 */
	public abstract NdbBlob getBlobHandle(long columnId)
			throws NdbApiException;
	@Deprecated
	public abstract NdbBlob getNdbBlobHandle(long columnId)
		throws NdbApiException;

	/**
	 * Gets a BlobHandle object using the String name for the Blob column in
	 * the the table. The BlobHandle object can be used to read/write the Blob.
	 * @param columnName name of the Column in the Schema
	 * @return NdbBlob object
     * @throws NdbApiException if there was a problem in the cluster when selecting this column.
	 */
	public abstract NdbBlob getBlobHandle(String columnName)
			throws NdbApiException;
	@Deprecated
	public abstract NdbBlob getNdbBlobHandle(String columnName)
		throws NdbApiException;

	/**
	 * TIMESTAMP_MAX is "2037-12-31 23:59:59", which is
	 * 2144361599000L in long UNIX format.  
     * <br>In sql/my_time.h, it's set to 0x7FFFFFFFL
	 */
	public static final long TIMESTAMP_MAX = 2144361599000L * 1000L;
//  <br>In Java format, it is (2144361599000L * 1000L), as the unit for Java Timestamp is nano-seconds, whereas the unit for UNIX Timestamp is milliseconds.
// Hence, we multiply the UNIX TIMESTAMP_MAX by 1000 to get the max Timestamp in Java.

	/**
	 * TIMESTAMP_MIN is the minimum Timestamp value, which is 0, or
	 * "1970-01-01 00:00:00".
	 */
	public static final long TIMESTAMP_MIN = 0L;


	/**
	 * Indicates that the primary key operation will read a row.
	 * @param mode the type of lock to take on the row (read last committed value, or take an exclusive lock)
     * @throws NdbApiException if there is a problem in the cluster.
	 */
	public abstract void readTuple(LockMode mode) throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val
    * @throws NdbApiException if there is a problem in the cluster.
	 */
	public void setShort(long columnId, short val) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column the value to be set for the column
	 * @throws NdbApiException
	 */
	public void setShort(String columnName, short val)
			throws NdbApiException;

    
	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val
     * @throws NdbApiException if there is a problem in the cluster.
	 */
	public void setInt(long columnId, int val) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column the value to be set for the column
	 * @throws NdbApiException
	 */
	public void setInt(String columnName, int val)
			throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setLong(long columnId, long val) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setLong(String columnName, long val)
			throws NdbApiException;



	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setFloat(long columnId, float val) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setFloat(String columnName, float val) throws NdbApiException;
	
	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setDouble(long columnId, double val) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setDouble(String columnName, double val) throws NdbApiException;
	

	/**
	 * Timestamp objects stored in the database lose nanosecond accuracy.
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param t if t is null, then setNull method is called for the column
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setTimestamp(long columnId, Timestamp t)
			throws NdbApiException;

	/**
	 * Timestamp objects stored in the database lose nanosecond accuracy.
	 * @param columnName name of the Column in the Schema
	 * @param t
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column. if columnName is null or empty string
	 * @throws NdbApiTimeStampOutOfBoundsException if Timestamp value is for year
	 * after 2037
	 */
	public void setTimestamp(String columnName, Timestamp t)
			throws NdbApiException, NdbApiTimeStampOutOfBoundsException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setBytes(long columnId, byte[] val)
			throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setBytes(String columnName, byte[] val)
			throws NdbApiException;

	/**
	 * This method first queries the cluster about the charset of columnName,
	 * before setting columnName to val.
	 * The supported charsets are Latin1 and utf8. If columnName has
	 * no charset, it is assume to be VARBINARY, and the charset is assumed to
	 * be latin1.
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setString(long columnId, String val)
			throws NdbApiException;

	/**
	 * This method first queries the cluster about the charsett of columnName,
	 * before setting columnName to val.
	 * The supported charsets are Latin1 and utf8. If columnName has
	 * no charset, it is assume to be VARBINARY, and the charset is assumed to
	 * be latin1.
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setString(String columnName, String val)
			throws NdbApiException;




	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setNull(long columnId) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public void setNull(String columnName) throws NdbApiException;




}