package com.mysql.cluster.ndbj;


/**
 * This class represents a connection to a cluster of data nodes.
 * A NdbClusterConnectionImpl object is used to bind to
 * a cluster. It is needed to create Ndb objects, that in turn can be used to
 * create a NdbTransaction and operation objects.
 * <p>
 * The close() method must be called on a NdbClusterConnectionImpl object
 * when it is no longer needed, to free underlying resources.
 * The isClosed() method can be used to test if a NdbClusterConnectionImpl object
 * has already been closed.
 * <p>
 * When creating or using a NdbClusterConnectionImpl object,
 * errors are manifested as NdbApiExceptions, for exaple, when methods are invoked on a closed NdbClusterConnectionImpl object.
 * <p>
 * More specialised exceptions that can be thrown are
 * NdbClusterConnectionPermanentException and NdbClusterConnectionTemporaryException that
 * denote a permanent problem when connecting to the cluster, and a temporary problem when connecting to the cluster, respectively.
 * In the case of a temporary error, you can retry  connecting to the cluster as it may succeed, for example, if the original problem was due to
 * a single node failure.
 * <br>
 * <br>Sample Code:<br>
 * <code>
 *      NdbClusterConnectionImpl connRef=null;<br>
 *      try {<br>
 *          connRef = NdbFactory.createNdbClusterConnection("ndb-connectstring");<br>
 *          // use the NdbClusterConnectionImpl object
 *      }<br>
 * 		catch (NdbClusterConnectionPermanentException e) {<br>
 * 			// Serious problem. Retrying connecting will not work.
 * 		}<br>
 * 		catch (NdbClusterConnectionTemporaryException e) {<br>
 * 			// We can retry connecting if we catch this exception
 * 		}<br>
 * 		catch (NdbApiException e) {<br>
 * 			// Base Exception
 * 		}<br>
 * 	    finally {<br>
 *          if (connRef != null)<br>
 *              connRef.close();<br>
 *	    }<br>
 *	</code>
 *
 *<p>
 * There is no restriction against instantiating multiple NdbClusterConnectionImpl objects representing connections to different management servers
 * in a single application, nor against using these for creating multiple instances of the Ndb class.
 * Such NdbClusterConnectionImpl objects (and the Ndb instances based on them) are not required even to connect to the same cluster.
 * <p>
 *  For example, it is entirely possible to perform application-level partitioning of data in such a manner that data
 *  meeting one set of criteria are "handed off" to one cluster via an Ndb object that makes use of an NdbClusterConnectionImpl
 *  object representing a connection to that cluster, while data not meeting those criteria
 *  (or perhaps a different set of criteria) can be sent to a different cluster through a different instance of Ndb
 *  that makes use of an NdbClusterConnectionImpl  "pointing" to the second cluster.
 *  <p>
 *  It is possible to extend this scenario to develop a single application that accesses an arbitrary number of clusters.
 *  However, in doing so, the following conditions and requirements must be kept in mind:
 *  <ul>
 *  <li>A cluster management server (ndb_mgmd) can connect to one and only one cluster without being restarted and reconfigured, as it must read the data telling it which data nodes make up the cluster from a configuration file (config.ini).
 *  <li>An NdbClusterConnectionImpl object "belongs" to a single management server whose hostname or IP address is used in instantiating this object (passed as the connectstring argument to its constructor); once the object is created,
 *  it cannot be used to initiate a connection to a different management server.
 *  <li>An Ndb object making use of this connection (NdbClusterConnectionImpl) cannot be re-used to connect to a different cluster management server
 *  (and thus to a different collection of data nodes making up a cluster). Any given instance of Ndb is bound to a
 *  specific NdbClusterConnectionImpl when created, and that NdbClusterConnectionImpl is in turn bound to a single and unique management server
 *  when it is instantiated.
 *  <li>The bindings described above persist for the lifetimes of the Ndb and NdbClusterConnectionImpl objects in question.
 *  </ul>
 *  <p>
 *  Therefore, it is imperative in designing and implementing any application that accesses multiple clusters in a single session,
 *  that a separate set of NdbClusterConnectionImpl and Ndb objects be instantiated for connecting to each cluster management server,
 *  and that no confusion arises as to which of these is used to access which MySQL Cluster.
 *  <p>
 *  It is also important to keep in mind that no direct "sharing" of data or data nodes between different clusters is possible. A data node can belong to
 *  one and only one cluster, and any movement of data between clusters must be accomplished on the application level.
 *
 * @see Ndb
 * @see NdbTransactions
 * @author jdowling
 */
public interface NdbClusterConnection
{
  

  /**
   * Connect to a cluster management server (ndb_mgmd)
   * @param retries specifies the number of retries to attempt
   * in the event of connection failure;
   * a negative value will result in the attempt to connect being repeated indefinitely
   * @param delay  specifies how often retries should be performed
   * @param verbose specifies if the method should print a report of its progess
   * @return 0 = success, 1 = recoverable error, -1 = non-recoverable error
   * @throws NdbClusterConnectionPermanentException if parameters are invalid (retries<0 or delay<0), or if there is a permanent problem in connecting to the management server.
   * @throws NdbClusterConnectionTemporaryException if there is a temporary problem in connecting to the management server. You can retry connct(), and it may succeed on the next attempt.
   */
  public abstract int connect(int retries, int delay, boolean verbose)
  	throws NdbClusterConnectionPermanentException, NdbClusterConnectionTemporaryException;

  /**
   * A client can set a name on the connection, which will be printed in the clusterlog.
   * Useful for identifying an NDB API client, e.g., for debugging.
   * @param name stringified name to be printed to clusterlog (e.g., name of the NDB API client)
   * @throws NdbApiException if the name parameter is null.
   */
  public abstract void setName(String name) throws NdbApiException;
  /**
   * Wait until the requested connection with one or more storage nodes is successful
   * @param time1 Number of seconds to wait until each node group has at least one live node
   * @param time2 Number of seconds to wait after all node groups have at least one valid node
   * @return = 0 all nodes live, > 0 at least one live node in each node group, < 0 error
   * @throws NdbClusterConnectionPermanentException if time1 < 0, time2 < 0, or if the native waitUntilReady function fails unexpectedly.  
   */
  public abstract int waitUntilReady(int time1, int time2) throws NdbApiException;


  /**
   * Factory method used to create a Ndb object using the NdbClusterConnectionImpl object.
   * Set the delayInit parameter to false to fully initialise the Ndb object. If 
   * you set delayInit to 'true', you need to call the init() method to fully initialise the Ndb object.
   * @param dirName is an optional parameter providing a namespace for the tables and indexes created in any connection from the Ndb object. The default value for this parameter is an empty string.
   * @param schemaName The optional schemaName provides an additional namespace for the tables and indexes created in a given catalog. The default value for this parameter is the string “def”. 
   * @param initThreads set to number of threads to pass to the init() method for the Ndb object 
   * @return Ndb object
   * @throws NdbApiException if a Ndb object cannot be created.
   */
  public abstract Ndb createNdb(String dirName, String schemaName, int initThreads) 
	throws NdbApiException;


  /**
   * Factory method used to create a Ndb object using the NdbClusterConnection object.
   * Set the delayInit parameter to false to fully initialise the Ndb object. If 
   * you set delayInit to 'true', you need to call the init() method to fully initialise the Ndb object.
   * @param dbName the name of the database for which we will use the Ndb object.
   * @param initThreads set to number of threads to pass to the init() method for the Ndb object 
   * @return Ndb object
   * @throws NdbApiException if a Ndb object cannot be created.
   */
  public abstract Ndb createNdb(String dbName, int initThreads) throws NdbApiException; 
  
  /**
   * Factory method used to create a Ndb object using the NdbClusterConnectionImpl object.
   * @param dbName the name of the database for which we will use the Ndb object.
   * @return Ndb object
   * @throws NdbApiException if a Ndb object cannot be created.
   */
  public abstract Ndb createNdb(String dbName) throws NdbApiException; 
  
  
  /**
   * The close method must be called on a NdbClusterConnection object when it
   * is no longer needed in order to free up its resources.
   * @return true if Ndb object closed successfully, false if Ndb object
   * has already been closed
   */
  public abstract boolean close();



}
