package com.mysql.cluster.ndbj;

import java.util.Map;


/**
 * NdbFactory is a class with a public static factory method for creating
 * a NdbClusterConnectionImpl object.
 * <br>NdbFactory is also used to instantiate all Ndb-accessing objects that 
 * are part of the pubic NDB/J API, that is, can be used by client apps.
 * 
 */
public class NdbFactory
{
		

  static {
    try {
      System.loadLibrary("ndbj");
    } catch (UnsatisfiedLinkError e) {
      System.err.println("Native code library failed to load. \n" + e);
      System.exit(1);
    }
    ndbj.ndb_init();
  }

    /**
     * This method is one of the first methods typically called in a NDB/J application. 
     * <br>It is a factory method used to create the top-level NdbClusterConnectionImpl object.
     * <br>Example Usage:
     * <code>
     *      <br>NdbClusterConnectionImpl conn=null;
     *      <br>try {
     *          <br>conn = NdbFactory.createNdbClusterConnection("ndb-connectstring");
     *          <br>// use the NdbClusterConnectionImpl object
     *      <br>}
     *      <br>catch (NdbClusterConnectionPermanentException e) {
     *      <br>// Serious problem. Retrying connecting will not work.
     *      <br>}
     * </code>
     * @param connectString the "--connectstring" used to connect to the cluster's management server (ndb_mgmd) 
     * @return a NdbClusterConnectionImpl object
     * @throws NdbApiException if a null string is passed in, or a connection object could not be created successfully.
     */
    static public NdbClusterConnection createNdbClusterConnection(String connectString)
    throws NdbApiException
    {
        return new NdbClusterConnectionImpl(connectString);
    }

    static public NdbClusterConnection createNdbClusterConnection()
    throws NdbApiException
    {
        return new NdbClusterConnectionImpl(null);
        
    }

 

    /**
     * Used internally by NDB/J.
     * @param op
     * @param ht
     * @return
     */
    static NdbResultSet createNdbResultSet(NdbBaseOperation op, Map<String,NdbRecAttr> ht)
    {
        return new NdbResultSetImpl(op, ht);
    }    
    
    /**
     * Used internally by NDB/J.
     * @param op
     * @param ht
     * @return
     */
    static NdbResultSet createNdbResultSet(NdbIndexScanOperation op, Map<String,NdbRecAttr> ht)
    {
        return new NdbResultSetImpl(op, ht);
        
    }    
    
    /**
     * Used internally by NDB/J.
     * @param op
     * @param ht
     * @return
     */
    static NdbResultSet createNdbResultSet(NdbScanOperation op, Map<String,NdbRecAttr> ht)
    {
        return new NdbResultSetImpl(op, ht);
       
    }        
 
}


