package com.mysql.cluster.ndbj;

import java.util.Calendar;
import java.math.BigInteger;


import com.mysql.cluster.ndbj.NdbOperation.LockMode;


/**
 * This interface describes operations that can be performed on an INDEX in a
 * table. <br>
 * Examples of index operations include:
 * <ul>
 * <li>equality (SQL-style: WHERE value='x'),
 * <li>greater-than (or equal-to) (SQL-style: WHERE value>'x')
 * <li>less-than (or equal-to) (SQL-style: WHERE value<'x')
 * </ul>
 * <br>
 * Equality operations on an index are set using operations
 * NdbBaseOperationEquals. <br>
 * The setBound methods are used to perform Greater-than and Less-than
 * operations on an index. <br>
 * The composition of bounds (SQL-style 'AND':, e.g., WHERE value>100 AND value
 * <200) are set by calling setBound methods multiple times.
 * <p>
 * Note: A bound is an exclusive operation. A bound excludes rows that would
 * otherwise be returned. <br>
 * For example, a greater-than bound excludes rows that are greater than the
 * supplied value. This means that a greater-than bound returns rows that are
 * less than the supplied value! <br>
 * If you are setting a single bound, then whereXY methods are SQL-style naming
 * that are just wrappers that call setBounds methods.
 * 
 * @see NdbBaseOperationEquals
 * @author jdowling
 */
public interface NdbIndexScanOperation extends NdbScanOperation,
		NdbBaseOperationEquals {

	/**
	 * The default name of the ordered index for the primary key.
	 */
	public static final String PRIMARY_INDEX_NAME = "PRIMARY";

	/**
	 * BoundType is used to specify bounds for an Index Scan using a
	 * NdbIndexScanOperation object. More than one bound may be specified on the
	 * NdbIndexScanOperation object. However, there is an restriction on using
	 * strict bounds (BoundLT and BoundGT). Strict bounds can only be used if it
	 * is the last bound that is specified on an NdbIndexScanOperation object.
	 * Non-strict bounds can be specified at any bound position (from first to
	 * last bound). <br>
	 * Another restriction on setbound is that when using a multi-part key with
	 * a BoundLE or BoundLE on the first key attribute a bound on the 2nd
	 * attribute that "points in the same direction" doesn't work. Only these
	 * combinations work: EQ/*, LE/L*, GE/GT* Any LT/* or GT/* combination does
	 * not work as documented, but LE/EQ, LE/GE, LE/GT and GE/EQ, GE/LE, GE/LT
	 * do not work, too.
	 * 
	 * BoundLT is a strict bound (Less than) BoundGT is a strict bound (greater
	 * than) BoundLE is a non-strict bound (less tha or equal to) BoundGE is a
	 * non-strict bound (greater than or equal to) BoundEQ is a non-strict bound
	 * (equal to)
	 * 
	 * In C++ NDB-API, these BoundTypes are defined in NdbIndexScanOperation.hpp
	 */
	public enum BoundType {
		/**
		 * Less than or equal to (<=). Non-strict bound.
		 */
		BoundLE(0), // /< lower bound
		/**
		 * Less than (<). Strict bound.
		 */
		BoundLT(1), // /< lower bound, strict
		/**
		 * Greater than or equal to (>=). Non-strict bound.
		 */
		BoundGE(2), // /< upper bound
		/**
		 * Greater than (>). Strict bound.
		 */
		BoundGT(3), // /< upper bound, strict
		/**
		 * Equal to (=). Non-strict bound.
		 */
		BoundEQ(4); // /< equality
		public int type;

		private BoundType(int type) {
			this.type = type;
		}
		
		public int swigValue() { 
			return this.type; 
		}
	};
	
	public void readTuples(LockMode lock_mode, long scan_flags, long parallel, long batch) 
		throws NdbApiException;
	
	public void readTuples(LockMode lock_mode, long scan_flags, long parallel) 
		throws NdbApiException;
	
	public void readTuples(LockMode lock_mode, long scan_flags) 
		throws NdbApiException;
	
	public void readTuples(LockMode lock_mode) 
		throws NdbApiException;
		
	public void readTuples() 
		throws NdbApiException;
	/**
	 * @param lock_mode
	 * @param batch
	 * @param parallel
	 * @param order_by
	 * @param order_desc
	 * @param read_range_no
	 * @param keyinfo
	 * @param multi_range
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#readTuples(com.mysql.cluster.ndbj.NdbOperation.LockMode, long, long, boolean, boolean, boolean, boolean, boolean)
	 */
	public void readTuples(LockMode lock_mode, long batch, long parallel, 
			boolean order_by, boolean order_desc, boolean read_range_no, 
			boolean keyinfo, boolean multi_range) 
	throws NdbApiException;

	/**
	 * @param lock_mode
	 * @param batch
	 * @param parallel
	 * @param order_by
	 * @param order_desc
	 * @param read_range_no
	 * @param keyinfo
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#readTuples(com.mysql.cluster.ndbj.NdbOperation.LockMode, long, long, boolean, boolean, boolean, boolean)
	 */
	public abstract void readTuples(LockMode lock_mode, long batch, long parallel, 
			boolean order_by, boolean order_desc, boolean read_range_no, boolean keyinfo) 
	throws NdbApiException;

	/**
	 * @param lock_mode
	 * @param batch
	 * @param parallel
	 * @param order_by
	 * @param order_desc
	 * @param read_range_no
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#readTuples(com.mysql.cluster.ndbj.NdbOperation.LockMode, long, long, boolean, boolean, boolean)
	 */
	public abstract void readTuples(LockMode lock_mode, long batch, long parallel, boolean order_by, 
			boolean order_desc, boolean read_range_no) 
	throws NdbApiException;

	/**
	 * @param lock_mode
	 * @param batch
	 * @param parallel
	 * @param order_by
	 * @param order_desc
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#readTuples(com.mysql.cluster.ndbj.NdbOperation.LockMode, long, long, boolean, boolean)
	 */
	public abstract void readTuples(LockMode lock_mode, long batch, long parallel, 
			boolean order_by, boolean order_desc) 
		throws NdbApiException;

	/**
	 * @param lock_mode
	 * @param batch
	 * @param parallel
	 * @param order_by
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#readTuples(com.mysql.cluster.ndbj.NdbOperation.LockMode, long, long, boolean)
	 */
	public abstract void readTuples(LockMode lock_mode, long batch, long parallel, boolean order_by) 
		throws NdbApiException;

	
	/**
	TODO: Make this version of the method
	 * An index scan operation to read many tuples.
	 * 
	 * @param lockMode
	 * @param batch
	 * @param parallel
	 * @param orderBy
	 * @param orderDesc
	 * @param readRangeNo
	 * @throws NdbApiException
	 
	public abstract void readTuples(LockMode lockMode, int batch, int parallel,
			boolean orderBy, boolean orderDesc, boolean readRangeNo)
			throws NdbApiException;
*/
	/**
	 TODO: Make this version of the method
	 * 
	 * @param lockMode
	 *            LM_Read Read with shared lock LM_Exclusive Read with exclusive
	 *            lock LM_CommittedRead Ignore locks; read last committed
	 * @param batch
	 *            The batch parameter specifies how many records will be
	 *            returned to the client from the server by the next
	 * @param parallel
	 *            The number of fragments to scan in parallel; use 0 to require
	 *            that the maximum possible number be used.
	 * @param orderBy
	 * @param orderDesc
	 * @param readRangeNo
	 * @param keyInfo
	 * @throws NdbApiException
	 *             if the object has been closed or if readTuples failed.
	 
	public abstract void readTuples(LockMode lockMode, int batch, int parallel,
			boolean orderBy, boolean orderDesc, boolean readRangeNo,
			boolean keyInfo) throws NdbApiException;
*/
	/**
	 * 
	 * @param columnId
	 * @param Type
	 * @param value
	 * @return
	 */
	public abstract void setBoundString(long columnId, BoundType Type, String value)
			throws NdbApiException;

	/**
	 * 
	 * @param columnName
	 * @param Type
	 * @param value
	 */
	public abstract void setBoundString(String columnName, BoundType Type,
			String value) throws NdbApiException;

	/**
	 * 
	 * @param columnId
	 * @param Type
	 * @param value
	 * @return int status code indicating success or failure
	 */
	public abstract void setBoundInt(long columnId, BoundType Type, int value)
			throws NdbApiException;

	/**
	 * 
	 * @param columnName
	 * @param Type
	 * @param value
	 * @return int status code indicating success or failure
	 */
	public abstract void setBoundInt(String columnName, BoundType Type, int value)
			throws NdbApiException;

	/**
	 * 
	 * @param columnId
	 * @param Type
	 * @param value
	 * @return int status code indicating success or failure
	 */
	public abstract void setBoundLong(long columnId, BoundType Type, long value)
			throws NdbApiException;

	/**
	 * 
	 * @param columnName
	 * @param Type
	 * @param value
	 * @return int status code indicating success or failure
	 */
	public abstract void setBoundLong(String columnName, BoundType Type, long value)
			throws NdbApiException;

	/**
	 * 
	 * @param columnId
	 * @param Type
	 * @param value
	 * @return
	 */
	public abstract void setBoundBytes(long columnId, BoundType Type, byte[] value)
			throws NdbApiException;

	/**
	 * 
	 * @param columnName
	 * @param Type
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract void setBoundBytes(String columnName, BoundType Type,
			byte[] value) throws NdbApiException;

/*	*//**
	 * 
	 * @param columnName
	 * @param value
	 * @return
	 *//*
	public abstract void whereGreaterThan(String columnName, String value)
			throws NdbApiException;

	*//**
	 * 
	 * @param columnName
	 * @param value
	 * @return
	 *//*
	public abstract void whereGreaterThanEqualTo(String columnName, String value)
			throws NdbApiException;

	*//**
	 * 
	 * @param columnName
	 * @param value
	 * @return
	 *//*
	public abstract void whereLessThan(String columnName, String value)
			throws NdbApiException;

	*//**
	 * 
	 * @param columnName
	 * @param value
	 * @return
	 *//*
	public abstract void whereLessThanEqualTo(String columnName, String value)
			throws NdbApiException;
*/
	/**
	 * @param anAttrName
	 * @param type
	 * @param anInputDateTime
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#setBound(long,
	 *      com.mysql.cluster.ndbj.NdbIndexScanOperation.BoundType,
	 *      java.util.Calendar)
	 */
	public void setBoundDatetime(String atAttrName, BoundType type, Calendar anInputDateTime)
			throws NdbApiException;

	/**
	 * @param anAttrName
	 * @param type
	 * @param val
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#setBound(long,
	 *      com.mysql.cluster.ndbj.NdbIndexScanOperation.BoundType, double)
	 */
	public void setBoundDouble(String atAttrName, BoundType type, double val)
			throws NdbApiException;

	/**
	 * @param anAttrName
	 * @param type
	 * @param val
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#setBound(long,
	 *      com.mysql.cluster.ndbj.NdbIndexScanOperation.BoundType, float)
	 */
	public void setBoundFloat(String atAttrName, BoundType type, float val)
			throws NdbApiException;
	/**
	 * @param anAttrId
	 * @param type
	 * @param anInputDateTime
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#setBound(long,
	 *      com.mysql.cluster.ndbj.NdbIndexScanOperation.BoundType,
	 *      java.util.Calendar)
	 */
	public void setBoundDatetime(long anAttrId, BoundType type, Calendar anInputDateTime)
			throws NdbApiException;

	/**
	 * @param anAttrId
	 * @param type
	 * @param val
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#setBound(long,
	 *      com.mysql.cluster.ndbj.NdbIndexScanOperation.BoundType, double)
	 */
	public void setBoundDouble(long anAttrId, BoundType type, double val)
			throws NdbApiException;

	/**
	 * @param anAttrId
	 * @param type
	 * @param val
	 * @return
	 * @throws NdbApiException
	 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation#setBound(long,
	 *      com.mysql.cluster.ndbj.NdbIndexScanOperation.BoundType, float)
	 */
	public void setBoundFloat(long anAttrId, BoundType type, float val)
			throws NdbApiException;
	
	
	public void endOfBound(long range_no) 
			throws NdbApiException;

	public void whereGreaterThan(long columnId, double val) throws NdbApiException;

	public void whereGreaterThan(String columnName, int val) throws NdbApiException;

	public void whereGreaterThan(long columnId, int val) throws NdbApiException;

	public void whereGreaterThan(String columnName, long val) throws NdbApiException;

	public void whereGreaterThan(long columnId, long val) throws NdbApiException;

	public void whereGreaterThan(String columnName, float val) throws NdbApiException;

	public void whereGreaterThan(long columnId, float val) throws NdbApiException;

	public void whereGreaterThan(long columnId, BigInteger val) throws NdbApiException;
	
	public void whereGreaterThanEqualTo(String columnName, int val) throws NdbApiException;

	public void whereGreaterThanEqualTo(long columnId, int val) throws NdbApiException;

	public void whereGreaterThanEqualTo(String columnName, long val) throws NdbApiException;

	public void whereGreaterThanEqualTo(long columnId, long val) throws NdbApiException;

	public void whereGreaterThanEqualTo(String columnName, double val) throws NdbApiException;

	public void whereGreaterThanEqualTo(long columnId, double val) throws NdbApiException;

	public void whereGreaterThanEqualTo(String columnName, float val) throws NdbApiException;

	public void whereGreaterThanEqualTo(long columnId, float val) throws NdbApiException;

	public void whereGreaterThanEqualTo(long columnId, BigInteger val) throws NdbApiException;

	public void whereLessThan(String columnName, int val) throws NdbApiException;

	public void whereLessThan(long columnId, int val) throws NdbApiException;

	public void whereLessThan(String columnName, long val) throws NdbApiException;

	public void whereLessThan(long columnId, long val) throws NdbApiException;

	public void whereLessThan(String columnName, double val) throws NdbApiException;

	public void whereLessThan(long columnId, double val) throws NdbApiException;

	public void whereLessThan(String columnName, float val) throws NdbApiException;

	public void whereLessThan(long columnId, float val) throws NdbApiException;

	public void whereLessThan(long columnId, BigInteger val) throws NdbApiException;

	public void whereLessThanEqualTo(String columnName, int val) throws NdbApiException;

	public void whereLessThanEqualTo(long columnId, int val) throws NdbApiException;

	public void whereLessThanEqualTo(String columnName, long val) throws NdbApiException;

	public void whereLessThanEqualTo(long columnId, long val) throws NdbApiException;

	public void whereLessThanEqualTo(String columnName, double val) throws NdbApiException;

	public void whereLessThanEqualTo(long columnId, double val) throws NdbApiException;

	public void whereLessThanEqualTo(String columnName, float val) throws NdbApiException;

	public void whereLessThanEqualTo(long columnId, float val) throws NdbApiException;

	public void whereLessThanEqualTo(long columnId, BigInteger val) throws NdbApiException;

	public void whereGreaterThan(String anAttrName, String anInputString) throws NdbApiException;

	public void whereGreaterThan(long anAttrId, String anInputString) throws NdbApiException;

	public void whereGreaterThanEqualTo(String anAttrName, String anInputString) throws NdbApiException;

	public void whereGreaterThanEqualTo(long anAttrId, String anInputString) throws NdbApiException; 
}