package com.mysql.cluster.ndbj;


/**
 * A NdbOperation object represents an operation using a primary key to access a row in a table.
 * <br>The main way to use it is to follow the following steps:
 * <ol>
 * <li>create a NdbOperation object using startTransaction method on a Ndb object
 * <li>decide on the type of operation to perfom on the operation (readTuple(), insertTuple(), updateTuple(), deleteTuple())
 * <li>define the search criteria for a row by setting the primary key using the equal() method.
 *      For example, if you are reading a tuple with a single column as a primary key, you execute the equal() method once on that column, 
 *      specifying the value of the column to indicate the row you are searching for.
 *     However, if your primary key consists of multiple columns (e.g., PRIMARY KEY(a, b, c)), then you call the equal() method for 
 *     all the of the columns in the primary key.<br>
 *     For examplee, for the following table: @code { create table t_multi (a int NOT NULL, b int NOT NULL, c char(20), PRIMARY KEY (a,b)) engine=ndb;) }
 *     You would call 
 *     @code { 
 *     operation.equal("a", VAL); 
 *     operation.equal("b",VAL2); 
 *     }.
 * <li>For any rows found after you called the equal() method(s), you can get/set the value of columns in those rows. These are known as <b>attribute actions</b>.
 * <li>Commit the operation by calling the exec() method on the NdbTransaction object.
 * <li>Retrieve any results using a NdbResultSet.
 * </ol>
 * <p>
 * The following code assumes that the operation is executed on the following table
 * <p>
 * <code> create table t (id int NOT NULL, val varchar(64), PRIMARY KEY id) engine=ndb;
 * </code>
 * <p>
 * <code>
 *          <br>trans = ndb.startTransaction();
 *          <br>NdbOperation op = trans.getNdbOperation(tableName);
 *          <br>op.readTuple(NdbOperation.LockMode.LM_Exclusive);
 *          <br>op.equal("id",PK_VAL);
 *          <br>op.getValue("val");
 *          <br>NdbResultSet rs = op.resultData();
 *          <br>trans.execute(NdbTransaction.ExecType.Commit,NdbTransaction.AbortOption.AbortOnError,1 );
 *          <br>while(rs.next())
 *          <br> String val = rs.getString(columnName);
 * </code>
 *
 * <p>An IllegalStateException is thrown if you attempt to invoke methods on this class
 * after the close method has been called on instances.
 *
 * @see com.mysql.cluster.ndbj.NdbBase
 * @see com.mysql.cluster.ndbj.NdbTransaction
 * @see com.mysql.cluster.ndbj.Ndb
 * @see com.mysql.cluster.ndbj.NdbClusterConnectionImpl
 * @see com.mysql.cluster.ndbj.NdbIndexOperation
 * @see com.mysql.cluster.ndbj.NdbScanOperation
 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation
 */
public interface NdbOperation extends NdbBaseOperationEquals, NdbInterpretedOperation {

    public enum LockMode {
        LM_Read(0), ///< Read with shared lock
        LM_Exclusive(1), ///< Read with exclusive lock
        LM_CommittedRead(2); ///< Ignore locks, read last committed value

        public int type = 0;

        public int swigValue() { 
        		return this.type;
        }
        private LockMode(int type) {
                this.type = type;
        }
    }
    
	/**
	 * Update a row in the table.
	 */
	public void insertTuple() throws NdbApiException;
	
	/**
	 * Update a row in the table.
	 */
	public void updateTuple() throws NdbApiException;
	
	public void writeTuple() throws NdbApiException;
	public void deleteTuple() throws NdbApiException;

	 public void interpretedUpdateTuple() throws NdbApiException;
	 public void interpretedDeleteTuple() throws NdbApiException;
}
