package com.mysql.cluster.ndbj;

import java.util.Map;
import java.sql.Timestamp;

/**
 * A NdbResultSet is used to store attributes in rows
 * that were selected after executing some query operation
 * on the cluster.
 *
 * You should be consistent in use of NdbResultSet objects.
 * Either access columns by name (String) or id (int).
 * Do not mix usage of access by name and id.
 * Id counts start at '0' (for the first column in a table).
 */
class NdbResultSetImpl implements NdbResultSet {
	/**
	 * We start reading results from '0'
	 */
	private Map<String,NdbRecAttr> resultSet;

	NdbBaseOperation op = null;


	private int scanCheck;

	private int lastColumnNull;

	private boolean run = false;

        private boolean fetched = false;

	private boolean hasData;

	private boolean rowsModified = false;


	/**
	 * Package-hidden constructor.
	 * <br> Use resultData() method on an operation object to create
	 * a NdbResultSet object.
	 * @param op
	 * @param ht
	 */
	NdbResultSetImpl(NdbBaseOperation op, Map<String,NdbRecAttr> ht) {
		resultSet = ht;
		this.op = op;
		hasData = true;
		scanCheck = 2;
		lastColumnNull = -1;
	}


	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getInt(int)
     */
	public int getInt(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getInt32();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getInt(java.lang.String)
     */
	public int getInt(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getInt32();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getShort(int)
     */
	public short getShort(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getShort();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getShort(java.lang.String)
     */
	public short getShort(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getShort();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getLong(int)
     */
	public long getLong(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getInt64();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getLong(java.lang.String)
     */
	public long getLong(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getInt64();
	}


	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getFloat(int)
     */
	public float getFloat(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getFloat();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getFloat(java.lang.String)
     */
	public float getFloat(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getFloat();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getDouble(int)
     */
	public double getDouble(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getDouble();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getDouble(java.lang.String)
     */
	public double getDouble(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getDouble();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getTimestamp(int)
     */
	public Timestamp getTimestamp(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getTimestamp();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getTimestamp(java.lang.String)
     */
	public Timestamp getTimestamp(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getTimestamp();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getBytes(int)
     */
	public byte[] getBytes(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getBytes();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getBytes(java.lang.String)
     */
	public byte[] getBytes(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getBytes();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getString(int)
     */
	public String getString(int columnId) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnId - 1);
		testNull(rec);
		return rec.getString();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getString(java.lang.String)
     */
	public String getString(String columnName) throws NdbApiException {
		NdbRecAttr rec = (NdbRecAttr) resultSet.get(columnName);
		testNull(rec);
		return rec.getString();
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getBlob(int)
     */
	public NdbBlob getBlob(int columnId) throws NdbApiException {
		return this.op.getBlobHandle(columnId);
	}
	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#getBlob(java.lang.String)
     */
	public NdbBlob getBlob(String columnName) throws NdbApiException {
		return this.op.getBlobHandle(columnName);
	}

        public boolean next(boolean fetchAllowed) throws NdbApiException {
	  // TODO: Seriously - this method and the next need to be redesigned
	  if (NdbScanOperationImpl.class.isInstance((Object)op))  {
	    int scanCheck = ((NdbScanOperation)op).nextResult(fetchAllowed);

	    return (scanCheck==0);
	  } else {
	    if (fetched) 
	      return false;
	    fetched=true;
	    return true;
                }
	  // It's not a scan operation
	}

	/**
	 * @see com.mysql.cluster.ndbj.NdbResultSet#next()
	 */
	public boolean next() throws NdbApiException {
		return next(true);
	}
	/*
		// TODO: Please double check me
		if (op == null) {
			throw new NdbApiException("NdbResultSet.next() "
					+ "- op == null");
		}
		// nextResult on NdbIndexScanOperation and NdbScanOperation return:
		// -1: Indicates that an error has occurred.
		// 0: Another tuple has been received.
		// 1: There are no more tuples to scan.
		// 2: There are no more cached records
		//   (invoke nextResult(true) to fetch more records).

		if (NdbScanOperationImpl.class.isInstance((Object)op) ||
				NdbIndexScanOperationImpl.class.isInstance((Object)op)) {

			scanCheck = ((NdbScanOperation)op).nextResult();
			
			// if nextResult(false) returns scanCheck == 0, transfer the record to another transaction.
			// if nextResult(false) returns scanCheck == 2, execute and commit the other transaction
			// Following this, call nextResult(true) — this fetches more
			// records and caches them in the NDB API.
			if (scanCheck == 2) {
				scanCheck = ((NdbScanOperation)op).nextResult(true, true);
				run = true;
			} else if (scanCheck == 0 && !run) {
				scanCheck = ((NdbScanOperation)op).nextResult(false, true);
			}
			if (scanCheck < 0) {
				throw new NdbApiException("problem with NdbResultSet.next()");
			}
			if (scanCheck == 0) {
				hasData = true;
			} else if (scanCheck == 2) {
				if (rowsModified) {
					 TODO: FIX This section - we're not storing an NdbTransaction within NdbScanOp any more 
					NdbTransaction trans = null;
					trans = ((NdbScanOperationImpl)theOp).getNdbTransaction();
					
					trans.execute(ExecType.NoCommit,
							AbortOption.AbortOnError, true);
					rowsModified = false; 
				}
				return next();
			} else if (scanCheck == 1) {
				hasData = false;
			}
			run = false;
			return hasData;
		} else {
			if (hasData) {
				hasData = false;
				return true;
			}
			return false;
		}
	}
*/
	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#deleteRow()
     */
	public void deleteRow() throws NdbApiException {

		if (!NdbScanOperationImpl.class.isInstance((Object)op)) {
			
			throw new NdbApiException("NdbResultSet::deleteRow(). "
					+ "Not implemented for PK operations.");
		}
		else if (op != null) {
			((NdbScanOperation)op).deleteCurrentTuple();
		}
		else {
			throw new NdbApiException("No underlying scan operation.");
		}
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#updateRow()
     */
	public NdbOperation updateRow() throws NdbApiException {
		if (!NdbScanOperationImpl.class.isInstance((Object)op)) {
			
			throw new NdbApiException("NdbResultSet::updateRow(). "
					+ "Not implemented for PK operations.");
		}
		else if (op != null) {
			NdbOperation theOp = ((NdbScanOperation)op).updateCurrentTuple();
			rowsModified = true;
			return theOp;
		}
		else {
			throw new NdbApiException("No underlying scan operation.");
		}
	}
	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#updateRow()
     */
	public NdbOperation updateRow(NdbTransaction trans) throws NdbApiException {
		if (!NdbScanOperationImpl.class.isInstance((Object)op)) {
			
			throw new NdbApiException("NdbResultSet::updateRow(). "
					+ "Not implemented for PK operations.");
		}
		else if (op != null) {
			NdbOperation theOp = ((NdbScanOperation)op).updateCurrentTuple((NdbTransactionImpl)trans);
			rowsModified = true;
			return theOp;
		}
		else {
			throw new NdbApiException("No underlying scan operation.");
		}
	}

	/**
     * @see com.mysql.cluster.ndbj.NdbResultSet#wasNull()
     */
	public boolean wasNull() throws NdbApiException {
		if (lastColumnNull == -1) {
			throw new NdbApiException(
					"No Column Read Before Calling NdbResultSet.wasNull()");
		}

		return (lastColumnNull > 0 ? true : false);
	}

	/**
	 * This private method tests to see if the last column read was NULL
	 * and sets a flag if true.
	 * It is not thread-safe.
	 * @param rec
	 */
	private void testNull(NdbRecAttr rec) throws NdbApiException {
		if (rec == null) {
			throw new NdbApiException("Couldn't find NdbRecAttrImpl for column. Have you called getXXX() on the column?");
		} else if (rec.isNULL()==-1) {
			throw new NdbApiException("Record is not open for reading");
		} else if (rec.isNULL()==1) {
			lastColumnNull = 1;
		} else {
			lastColumnNull = 0;
		}
	}

}

