package com.mysql.cluster.ndbj;

import java.sql.Timestamp;
import java.util.Calendar; 
import java.math.BigInteger; 


/**
 * NdbScanFilterRef provides an alternative means of specifying
 * filters for scan operations.
 * <br> Note: This class is still under development and
 * is likely to change in future releases.
 *
 * <br>You must call methods in the following order to define a "Node" in a tree of Scan Filters:
 * (1) call begin() method (to define a logical operation (AND, OR) on filters)
 * (2) define your filter using one of the cmp(), eq(), ne(), etc.
 * (3) call end() method
 *
 * You can nest "Nodes" (consisting of begin(), filters, end()) in logical
 * operations, e.g.:
 * <br>
 * <code>
 * <br>try {
 * <br>NdbScanFilterRef sf1 = scanOp.getNdbScanFilter();
 * <br>sf1.begin(NdbScanFilterRef.Group.OR); // Root
 * <br>sf1.eq("colX", 33); //"Node 1"
 * <br>sf1.eq("colY", 66); //"Node 2"
 * <br>sf1.end();
 * <br>}
 * <br>catch (NdbApiException e) {...}
 * </code>
 */
public interface NdbScanFilter {


	/**
	 * A ScanFilter can have different conditions added to it to make a compound.
	 * <br> Conditions can be combined into a compound using the following logical operations:
	 * AND, OR, NAND and NOR.
	 */
	 public enum Group {
		    AND(ndbjJNI.NdbScanFilterImpl_AND_get()),
		    OR(ndbjJNI.NdbScanFilterImpl_OR_get()),
		    NAND(ndbjJNI.NdbScanFilterImpl_NAND_get()),
		    NOR(ndbjJNI.NdbScanFilterImpl_NOR_get());

		    public final int swigValue() {
		      return swigValue;
		    }

		    public static Group swigToEnum(int swigValue) {
		      Group[] swigValues = Group.class.getEnumConstants();
		      if (swigValue < swigValues.length && swigValue >= 0 && swigValues[swigValue].swigValue == swigValue)
		        return swigValues[swigValue];
		      for (Group swigEnum : swigValues)
		        if (swigEnum.swigValue == swigValue)
		          return swigEnum;
		      throw new IllegalArgumentException("No enum " + Group.class + " with value " + swigValue);
		    }

		    @SuppressWarnings("unused")
		    private Group() {
		      this.swigValue = SwigNext.next++;
		    }

		    @SuppressWarnings("unused")
		    private Group(int swigValue) {
		      this.swigValue = swigValue;
		      SwigNext.next = swigValue+1;
		    }

		    @SuppressWarnings("unused")
		    private Group(Group swigEnum) {
		      this.swigValue = swigEnum.swigValue;
		      SwigNext.next = this.swigValue+1;
		    }

		    private final int swigValue;

		    private static class SwigNext {
		      private static int next = 0;
		    }
		  }



	/**
	 * COND_LE: condition less than or equal to (<=)
	 * COND_LT: condition less than (<)
	 * COND_GE: condition greater than or equal to (>=)
	 * COND:GT: condition greater than (>)
	 * COND_EQ: condition equal to (=)
	 * COND_NE: condition not equal to (!=)
	 * COND_LIKE: Pattern match for Strings using specified String.
	 * 			Similar to 'like' SQL operator, e.g., SELECT * FROM TABLE where colX LIKE 'h%';
	 * 			<br> This will match all rows where column, colX, has a String starting with the letter 'h'.
	 *
	 * COND_NOT_LIKE:
	 */
	  public enum BinaryCondition {
		    COND_LE(ndbjJNI.NdbScanFilterImpl_COND_LE_get()),
		    COND_LT(ndbjJNI.NdbScanFilterImpl_COND_LT_get()),
		    COND_GE(ndbjJNI.NdbScanFilterImpl_COND_GE_get()),
		    COND_GT(ndbjJNI.NdbScanFilterImpl_COND_GT_get()),
		    COND_EQ(ndbjJNI.NdbScanFilterImpl_COND_EQ_get()),
		    COND_NE(ndbjJNI.NdbScanFilterImpl_COND_NE_get()),
		    COND_LIKE(ndbjJNI.NdbScanFilterImpl_COND_LIKE_get()),
		    COND_NOT_LIKE(ndbjJNI.NdbScanFilterImpl_COND_NOT_LIKE_get());

		    public final int swigValue() {
		      return swigValue;
		    }

		    public static BinaryCondition swigToEnum(int swigValue) {
		      BinaryCondition[] swigValues = BinaryCondition.class.getEnumConstants();
		      if (swigValue < swigValues.length && swigValue >= 0 && swigValues[swigValue].swigValue == swigValue)
		        return swigValues[swigValue];
		      for (BinaryCondition swigEnum : swigValues)
		        if (swigEnum.swigValue == swigValue)
		          return swigEnum;
		      throw new IllegalArgumentException("No enum " + BinaryCondition.class + " with value " + swigValue);
		    }

		    @SuppressWarnings("unused")
		    private BinaryCondition() {
		      this.swigValue = SwigNext.next++;
		    }

		    @SuppressWarnings("unused")
		    private BinaryCondition(int swigValue) {
		      this.swigValue = swigValue;
		      SwigNext.next = swigValue+1;
		    }

		    @SuppressWarnings("unused")
		    private BinaryCondition(BinaryCondition swigEnum) {
		      this.swigValue = swigEnum.swigValue;
		      SwigNext.next = this.swigValue+1;
		    }

		    private final int swigValue;

		    private static class SwigNext {
		      private static int next = 0;
		    }
		  }

	/**
	 * The begin() method starts the definition of a scan filter.
	 * Define your scan filters by calling methods such as cmp(),
	 * eq() on your NdbScanFilterRef object.
	 * Use end() method to define the end of a scan filter.
	 * Note: if you call getValue() methods on the NdbScanOperationRef object
	 * between calling begin() and end() on a NdbScanFilterRef, the getValue()
	 * method will not have any effect. Make sure to call getValue() on your
	 * NdbScanOperationRef object before you define any NdbScanFilterRef objects.
	 *
	 * Preconditions: Valid NdbScanOperationRef, NdbTransactionRef, NdbRef, NdbClusterConnectionImpl Objects
	 *
	 * @param group
	 * @throws NdbApiException
	 */
	public abstract int begin(Group group) throws NdbApiException;

	/**
	 * The begin() method starts the definition of a scan filter.
	 * Define your scan filters by calling methods such as cmp(),
	 * eq() on your NdbScanFilterRef object.
	 * Use end() method to define the end of a scan filter.
	 * Note: if you call getValue() methods on the NdbScanOperationRef object
	 * between calling begin() and end() on a NdbScanFilterRef, the getValue()
	 * method will not have any effect. Make sure to call getValue() on your
	 * NdbScanOperationRef object before you define any NdbScanFilterRef objects.
	 *
	 * Preconditions: Valid NdbScanOperationRef, NdbTransactionRef, NdbRef, NdbClusterConnectionImpl Objects
	 *
	 * @throws NdbApiException
	 */
	public abstract int begin() throws NdbApiException;
	
	/**
	 * Call end() method to finish the definition of a scan filter.
	 * Preconditions: You must have called begin() method on the NdbScanFilterRef object.
	 * @throws NdbApiException
	 */
	public abstract int end() throws NdbApiException;

	/**
	 * The compare filter for String values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type String).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid, the column type is not VARBINARY or CHAR/VARCHAR
	 * with character set utf-8 or latin-1.
     * @see BinaryCondition
	 */
	public abstract int cmpString(BinaryCondition cond, int columnId, String val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmpString(BinaryCondition cond, String columnName, String val)
			throws NdbApiException;

	/**
	 * The compare filter for NULL values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type String).
	 * @param cond a BinaryCondition type
	 * @return 
	 * @throws NdbApiException 
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName)
			throws NdbApiException;

	/**
	 * The compare filter for int values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type int).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, int val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, int val)
			throws NdbApiException;

	/**
	 * The compare filter for long values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type long).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, long val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, long val)
			throws NdbApiException;

	/**
	 * The compare filter for double values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type double).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, double val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, double val)
			throws NdbApiException;

	/**
	 * The compare filter for float values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type float).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, float val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, float val)
			throws NdbApiException;

	/**
	 * The compare filter for int values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type int).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmpTimestamp(BinaryCondition cond, int columnId, Timestamp val)
			throws NdbApiException;

	/**
	 * TODO: Fix this naming problem
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmpTimestamp(BinaryCondition cond, String columnName, Timestamp val)
			throws NdbApiException;

	/**
	 * The compare filter for int values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type int).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, Calendar val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, Calendar val)
			throws NdbApiException;
	/**
	 * The compare filter for int values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type int).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, byte[] val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, byte[] val)
			throws NdbApiException;

	/**
	 * The compare filter for int values.
	 * This method filters all rows using the column (specified by columnId)
	 * and value (val of type int).
	 * @param cond a BinaryCondition type
	 * @param val
	 * @return 
	 * @throws NdbApiException if the parameters are invalid,
     * @see BinaryCondition
	 */
	public abstract int cmp(BinaryCondition cond, int columnId, BigInteger val)
			throws NdbApiException;

	/**
	 *
	 * @param cond
	 * @param columnName
	 * @param val
	 * @return
	 * @throws NdbApiException
	 */
	public abstract int cmp(BinaryCondition cond, String columnName, BigInteger val)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where
	 * the value of an entry in the supplied column number is <b>equal</b> to the supplied int value.
	 * @param columnId column number
	 * @param value integer used to filter out rows from the scan
	 * @throws NdbApiException if filtering failed in cluster
     * @throws NdbApiRuntimeException if the columnId is invalid
	 */
	public abstract int eq(int columnId, BigInteger value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where
	 * the value of an entry in the supplied column name is <b>equal</b> to the supplied int value.
	 *
	 * @param columnName column name
	 * @param value integer used to filter out rows from the scan
	 * @throws NdbApiException if filtering failed in the cluster
     * @throws NdbApiRuntimeException if the column name is invalid
	 */
	public abstract int eq(String columnName, BigInteger value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where
	 * the supplied columnId is equal to the supplied long value.
	 *
	 * @param columnId column number
	 * @param value long used to filter out rows from the scan
	 * @throws NdbApiException if columnId is not defined, or if filtering failed in the cluster
	 */
	public abstract int eq(int columnId, long value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where
	 * the supplied column name is equal to the supplied long value.
	 *
	 * @param columnName the name of the column
	 * @param value long used to filter out rows from the scan
	 * @throws NdbApiException if filtering failed in the cluster.
     * @throws NdbApiRuntimeException if the column name is invalid
	 */
	public abstract int eq(String columnName, long value)
			throws NdbApiException;

	/**
	 * Wrapper method to filter rows where columnId has VARCHAR,
	 * VARBINARY or CHAR with supplied value value.
	 * @param columnId column number
	 * @param value String used to filter out rows from the scan
	 * @throws NdbApiException if columnId is not defined, or if filtering failed in the cluster, or if charset for column not latin1 or utf8
	 */
	public abstract int eq(int columnId, String value) throws NdbApiException;

	/**
	 * TODO: Fix naming problem here
	 * Wrapper method to filter rows where columnName has VARCHAR,
	 * VARBINARY or CHAR with supplied value.
	 *
	 * @param columnName column name
	 * @param value String used to filter out rows from the scan
	 * @throws NdbApiException if filtering failed
     * @throws NdbApiRuntimeException if the column name is invalid or if charset for column not latin1 or utf8
	 */
	public abstract int eq(String columnName, String value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>not equal</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ne(int columnId, BigInteger value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>not equal</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ne(String columnName, BigInteger value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>not equal</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ne(int columnId, long value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>not equal</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ne(String columnName, long value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>less than</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int lt(int columnId, BigInteger value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>less than</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int lt(String columnName, BigInteger value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>less than</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int lt(int columnId, long value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>less than</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int lt(String columnName, long value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>less than or equal to</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int le(int columnId, BigInteger value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>less than or equal to</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int le(String columnName, BigInteger value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>less than or equal to</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int le(int columnId, long value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>less than or equal to</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int le(String columnName, long value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>greater than</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int gt(int columnId, BigInteger value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>greater than</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int gt(String columnName, BigInteger value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>greater than</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int gt(int columnId, long value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>greater than</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int gt(String columnName, long value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>greater than or equal to</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ge(int columnId, BigInteger value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>greater than or equal to</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ge(String columnName, BigInteger value)
			throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column number are <b>greater than or equal to</b> to the supplied value.
	 *
	 * @param columnId
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ge(int columnId, long value) throws NdbApiException;

	/**
	 * Filters rows returned from scan; returns rows where values in
	 * the supplied column name are <b>greater than or equal to</b> to the supplied value.
	 *
	 * @param columnName the name of the column
	 * @param value
	 * @throws NdbApiException
	 */
	public abstract int ge(String columnName, long value)
			throws NdbApiException;

	/**
	 * Check if a column, idenitifed by the column id number, is null.
	 * @param columnId column number
	 * @return
	 * true: if the value of columnName is Null;
	 * <br>false: if the value of columnName is not Null;
	 * @throws NdbApiException
	 */
	public abstract int isNull(int columnId) throws NdbApiException;

	/**
	 * Check if a column, idenitifed by the column name, is null.
	 *
	 * @param columnName name of the column
	 * @return
	 * true: if the value of columnName is Null;
	 * <br>false: if the value of columnName is not Null;
	 * @throws NdbApiException if columnName is not defined for the NdbScanOperationRef
	 */
	public abstract int isNull(String columnName) throws NdbApiException;

	/**
	 * TODO: Fix Docs here, they are wrong
	 * Check if a column, idenitifed by the column id number, is not null
	 * @param columnId column number
	 * @return
	 * true: if the value of columnName is not Null;
	 * <br>false: if the value of columnName is Null;
	 * @throws NdbApiException if columnName is not defined for the NdbScanOperationRef
	 */
	public abstract int isNotNull(int columnId) throws NdbApiException;

	/**
	 * Check if a column, idenitifed by the column name, is not null
	 *
	 * @param columnName name of the column
	 * @return
	 * true: if the value of columnName is not Null;
	 * <br>false: if the value of columnName is Null;
	 * @throws NdbApiException if columnName is not defined for the NdbScanOperationRef
	 */
	public abstract int isNotNull(String columnName) throws NdbApiException;

}