package com.mysql.cluster.ndbj;

import java.math.BigInteger;

/**
 * An NdbTransaction object represents a transaction that can be used
 * to execute a batch of operations in the database.
 * <p>The NdbTransaction object must be closed independent of its outcome,
 * i.e. even if there is an error. This can be done by calling the method
 * close on the NdbTransaction object or by calling the method closeTransaction
 * on the Ndb object used to create the transaction.
 * <p>
 * A transaction (represented by an NdbTransaction object) belongs to
 * an Ndb object and is created calling the startTransaction method on a Ndb object.<br>
 * A transaction consists of a number of operations (represented by NdbOperation, NdbScanOperation, NdbIndexOperation, and NdbIndexScanOperation objects),
 * which can be executed in the cluster. Each operation accesses exactly one table.
 * <p>After getting the NdbTransaction object, the first step is to get (allocate) an operation given the table name using one of the
 * methods getNdbOperation, getNdbScanOperation, getNdbIndexOperation, or getNdbIndexScanOperation.
 * Then the operation is defined. Several operations can be defined on the same NdbTransaction object,
 * they will in that case be executed in parallel. When all operations are defined, the execute method
 * sends them to the NDB kernel for execution.
 *
 * <p>An IllegalStateException is thrown if you attempt to invoke methods on this class
 * after the close method has been called on instances.
 *
 * 
 * <br>Sample Code:<br>
 * <code>
 *      NdbClusterConnectionImpl conn=null;<br>
 *      Ndb ndb=null;<br>
 *      NdbTransaction trans=null;<br>
 *      try {<br>
 *          conn = NdbFactory.createNdbClusterConnection("ndb-connectstring");<br>
 *          ndb = conn.createNdb("database");<br>
 *          ndb.init(12); // Max num of concurrent transactions is set to 12 <br>
 *          trans = ndb.startTransaction();<br>
 *          // use the transaction
 *      }<br>
 *      catch (NdbApiException e) {<br>
 *          // Handle Error
 *      }<br>
 *      finally {<br>
 *          // You must close the ndb object to free up native resources.<br>
 *          if  (trans != null)<br>
 *              trans.close();<br>
 *          if  (ndb != null)<br>
 *              ndb.close();<br>
 *          if (conn != null)<br>
 *              conn.close();<br>
 *      }<br>
 *</code>
 * 
 * @see com.mysql.cluster.ndbj.NdbBase
 * @see com.mysql.cluster.ndbj.Ndb
 * @see com.mysql.cluster.ndbj.NdbClusterConnectionImpl
 * @see com.mysql.cluster.ndbj.NdbOperation
 * @see com.mysql.cluster.ndbj.NdbIndexOperation
 * @see com.mysql.cluster.ndbj.NdbScanOperation
 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation
 * 
 * @author jdowling
 */
public interface NdbTransaction {

	

	    /**
	     * This type is used to determine the state of the transaction.
	     */
	    public enum ExecutionState
	    {
	    	/**
	    	 * The transaction has been started.
	    	 */
	      STARTED,
	      /**
	       * The transaction has been executed but not committed.
	       */
	      EXECUTED_NOT_COMMITTED,
	      /**
	       * The transaction has been executed and committed.
	       */
	      EXECUTED_COMMITTED,
	      /**
	       * The transaction has been rolled back.
	       */
	      EXECUTED_ROLLED_BACK,
	      /**
	       * The transaction has been closed.
	       */
	      CLOSED,
	      /**
	       * The transaction is in an invalid state.
	       */
	      ERROR;

	      private ExecutionState()
	      {
	      }
	    };


		/**
		 * The transaction stores a listt of Blob objects for the transactions. 
         * Used internally in NDB/J.
		 * @param blob object
		 
	    abstract void addNdbBlob(NdbBlob blob);*/
	/**
	 * This method executes the transaction and all the operations that have been added it.
	 * It either succeeds or fails.
	 *
	 * The execute method returns when the NDB kernel has completed execution of all operations defined
	 * before the call to execute. All allocated operations should be properly defined before calling execute.
	 * A call to the execute method uses one out of three types of execution: Commit, NoCommit, or Rollback.
	 * If an operation in the transaction fails, it is aborted if abortOption is set
	 * to NdbTransaction.AbortOption.AbortOnError , and the transaction continues excution after the failed
	 * operation if abortOption is NdbTransaction.AbortOption.AO_ignoreError.
	 * <p>If forceSend is set to 0, the transaction is placed in a send buffer for up to a maximum of 10 milliseconds,
	 * after which it is flushed to the cluster. This can help improve system performance in high load, through increased
	 * batching of transactions. If forceSend is set to 1, the transaction is flushed
	 * immediately to the cluster. This improves transaction performance, but may degrade system scalability (throughput of
	 * the cluster in terms of number of transactions that can be executed per time unit).
	 *
	 * @param execType specifies whether the transaction should be committed, not-committed or rolled-back.
	 * <br>NdbTransactionImpl.ExecType.NoCommit : executes operations without committing them.
	 * <br>NdbTransactionImpl.ExecType.Commit : executes remaining operation and commits the complete transaction
	 * <br>NdbTransactionImpl.ExecType.Rollback : rollbacks the entire transaction.
	 * @param abortOption specifies whether the transaction should be aborted or not if one of its operations fails.
	 * <br>NdbTransactionImpl.AbortOption.AbortOnError : (default) The transaction is aborted if there are any error during the execution
	 * <br>NdbTransactionImpl.AbortOption.AO_IgnoreError : Continue execution of transaction even if operation fails
	 * @param force
	 * <br>true : put the transaction in the send buffer
	 * <br>false : flush the send buffer so that the transaction executes immediately (improves transaction performance, may degrade system performance)
	 * @throws NdbApiException
	 * <br> base exception. Catch this if you don't care about retrying transactions that failed because of temporary problems in the cluster.
	 * @throws NdbApiTemporaryException
	 * <br> transaction failed. If this exception is caught, retry the transaction. The transaction failed due a temporary problem (e.g., a single node failed),
	 * and retrying the transaction may cause it to succeed.
	 * @throws NdbApiPermanentException
	 * <br> transaction failed. if this exception is caught, the transaction should not be retried, as it failed due to some serious error
	 * @throws NdbApiUserAndPermanentException
	 * <br> transaction failed. if this exception is caught, the transaction should not be retried, as it failed due to some serious (possibly user) error (such as deleting a row that doesn't exist).
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public int execute(ExecType execType, AbortOption abortOption, boolean force) throws NdbApiException;
	public int execute(ExecType execType, AbortOption abortOption) throws NdbApiException;
	public int execute(ExecType execType) throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param tableName name of the schema
	 * @return a NdbOperation object
	 * <br> is thrown if the cluster could not create an IndexScanOperation (e.g., there are no Operation records available due to too heavy load on the cluster)
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public NdbOperation getNdbOperation(String tableName)
			throws NdbApiException;

	/**
	 * Creates a NdbIndexOperation (operation on a unique index) and adds it to the transaction
	 * @param indexName the String name for the index (use "show create table TABLE_NAME\G" to find the index name)
	 * @param tableName name of the schema
	 * @return a NdbIndexOperation object
	 * <br> is thrown if the cluster could not create an IndexScanOperation (e.g., there are no Operation records available due to too heavy load on the cluster)
	 * @throws NdbApiException if tableNameis null, the object has been closed, or the Operation object could not be created
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */

	public NdbIndexOperation getNdbIndexOperation(String indexName, String tableName) throws NdbApiException;
	public NdbIndexOperation getNdbIndexOperation(NdbIndex theIndex) throws NdbApiException;

	/**
	 * Creates a NdbIndexScanOperation (operation on an index) and adds it to the transaction.
	 * @param indexName the String name for the index (use "show create table TABLE_NAME\G" to find the index name)
	 * @param tableName name of the schema
	 * @return a NdbIndexScanOperation object
	 * @throws NdbApiException if the indexName or tableName was null or a NdbIndexScanOperation object could not be created
	 * <br> is thrown if the cluster could not create an IndexScanOperation (e.g., there are no Operation records available due to too heavy load on the cluster)
	 * @throws NdbApiException if tableName or indexName is null or empty, the object has been closed, or the Operation object could not be created
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public NdbIndexScanOperation getNdbIndexScanOperation(String indexName, String tableName) throws NdbApiException;

	public NdbIndexScanOperation getNdbIndexScanOperation(
			NdbIndex theIndex) throws NdbApiException;

	/**
	 * Creates a NdbScanOperation and adds it to the transaction. A NdbScanOperation can be
	 * used to perform a table scan, where all of the rows in a table are selected/updated/deleted.
	 *
	 * <br>Call the close() method on the NdbTransaction object when you have finished using
	 * it to free JNI resourcs.
	 * @param tableName name of the schema
	 * @return NdbScanOperation object
	 * @throws NdbApiException if the tableName was null or a NdbScanOperation object could not be created
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public NdbScanOperation getNdbScanOperation(String tableName)
			throws NdbApiException;

	/**
	 * This method should only be used after the transaction has been executed,
	 * but before the transaction has been closed.
	 *
	 * Transactions consist of groups of operations. When a transaction has
	 * been executed, you can call this method to get
	 * the next completed operation in a transaction after
	 * the supplied operation object. If null is specified as a parameter,
	 * then the first completed operation object is returned.
	 *
	 * @param op an operation object in the transaction. If op is null, the
	 * first completed operation in the transaction is returned.
	 * @return the next completed operation object in the transaction.
	 * @throws NdbApiException if there is an error in the JNI library
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public NdbOperation getNextCompletedOperation(NdbOperationImpl op)
			throws NdbApiException;

	/**
	 * closes the NdbTransaction object. The object can no longer be used after the close()
	 * method has been invoked. You have to create a new NdbTransaction object using
	 * a Ndb.startTransaction() method.
	 */
	public void close();

	/**
	 * checks if the transaction object is closed
	 * @return true if closed, false if not closed
	 */
	public boolean isClosed();

	/**
	 * Returns the number of rows in the supplied table name.
	 * @param tableName name of Schema
	 * @return number of rows in table
	 * @throws NdbApiException
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public BigInteger selectCount(String tableName) throws NdbApiException;

	/**
	 * restart the transaction.
	 * This should not be called after the transaction has been closed.
	 * @return
	 * @throws NdbApiException
	 * @throws IllegalStateException if the close method has already been called on the object.
	 */
	public int restart() throws NdbApiException;
	/**
	 * @param execType
	 * @param cb
	 * @param abortOption
	 * @see NdbTransaction#executeAsynchPrepare(ExecType,
	 *      BaseCallback,
	 *      AbortOption)
	 */
	public void executeAsynchPrepare(ExecType execType, BaseCallback cb, AbortOption abortOption);
	/**
	 * @param execType
	 * @param cb
	 * @see NdbTransaction#executeAsynchPrepare(ExecType,
	 *      BaseCallback)
	 */
	public void executeAsynchPrepare(ExecType execType, BaseCallback cb);
	public NdbOperationImpl getNdbOperation(NdbTable aTable) throws NdbApiException;
	public NdbIndexScanOperationImpl getNdbIndexScanOperation(NdbIndex anIndex, NdbTable aTable) throws NdbApiException;
	public NdbIndexOperationImpl getNdbIndexOperation(NdbIndex anIndex, NdbTable aTable) throws NdbApiException;
	public NdbScanOperationImpl getNdbScanOperation(NdbTable aTable) throws NdbApiException;

}