package com.mysql.cluster.ndbj.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.util.Properties;

import com.mysql.cluster.ndbj.NdbApiException;

/**
 * 
 * The NdbConfigurator is a utility class that can be used
 * to help configure a cluster using a properties file.
 * <br>Usage:
 * <code>
 * <br>NdbConfigurator.loadParamsFromConfigFile("ndbj.props");
 * <br>...
 * <br>NdbClusterConnectionRef conn = NdbFactoryRef.createNdbClusterConnection(NdbConfigurator.CONNECSTRING);
 * <br>NdbRef ndb= con.createNdb(NdbConfigurator.DB_NAME); 
 * <br>
 * <br>NdbConfigurator.createTable("......");
 * @author jdowling
 */
public class NdbConfigurator
{
	static String CONFIG_FILENAME="";
	static String CONNECTSTRING="";
	static String LIB_PATH = "";
	static String DB_NAME="";
	static String MODE="development";

	/**
	 * HTTP Proxy Settings
	 */
    protected static String proxyHost 		= null;
    protected static int 	port        	= 8080;
	protected static String proxyUsername	= null;
	protected static String proxyPassword	= null;



	static void loadParamsFromConfigFile(String configFileName)
	throws NdbApiException
	{
		CONFIG_FILENAME=configFileName;
		File f =null;
		try {
			InputStream in=null;
			f = new File(CONFIG_FILENAME);
			if (f.exists()){
				if (f.canRead()){
					in = new FileInputStream(f);
					}
				else {
					System.err.println("Could not open " + CONFIG_FILENAME);
					System.exit(-1);
					}
				}
			else {
				System.err.println("Could not find " + CONFIG_FILENAME);
				System.exit(-1);
				}
		    loadProperties(in);
		}
		catch (java.io.FileNotFoundException e) {
			System.err.println(e.getMessage());
		}
		catch (NdbApiException e) {
			System.out.print(e.getStackTrace());
			System.err.println(e.getMessage());
		}
	}


	/**
	 * Loads parameters from a local file ("config/crl.props")
	 * Can be overriden - e.g., getting the parameters from a central server
	 * @throws CRLParamsException
	 */
	void loadParamsFromURL(String url) throws NdbApiException {
		try {
			URL server = new URL(url);

			if (proxyUsername != null && proxyPassword != null) {
				Authenticator.setDefault(new SimpleAuthenticator(
						proxyUsername,proxyPassword));
//				if (proxy != null) {
//					dnsWorkAround();
//				}
			}

			Properties systemProperties = System.getProperties();
			if (proxyHost != null) {
				systemProperties.setProperty("http.proxyHost",proxyHost);
				systemProperties.setProperty("http.proxyPort",Integer.toString(port));
			}
			HttpURLConnection connection = (
				    HttpURLConnection)server.openConnection();
			connection.connect();

			InputStream in = connection.getInputStream();
			// set up new properties object from file "myProperties.txt"

			BufferedInputStream bufIn= new BufferedInputStream(in);

		    Properties p = new Properties(System.getProperties());
		    p.load(bufIn);

	        System.setProperties(p);
	        System.getProperties().list(System.out);

		}
		catch (MalformedURLException e) {
			throw new NdbApiException(e.toString());
		}
		catch (IOException e) {
			throw new NdbApiException(e.toString());
		}
		catch (NullPointerException e) {
			throw new NdbApiException(e.toString());
		}
		catch (IllegalArgumentException e) {
			throw new NdbApiException(e.toString());
		}
		catch (SecurityException e) {
			throw new NdbApiException(e.toString());
		}
	}

	static void setHttpProxy(String hostname, int portNumber, String username, String password) {
		proxyHost=hostname;
		port=portNumber;
		proxyUsername=username;
		proxyPassword=password;
	}

	static void loadProperties(InputStream in)
	throws NdbApiException {

		boolean libPathSet=true;
		Properties p = new Properties(System.getProperties());

		try {
			p.load(in);

			// set the system properties
	        System.setProperties(p);

			NdbConfigurator.CONNECTSTRING= System.getProperty("ndbj.connectstring");
			if (NdbConfigurator.CONNECTSTRING == null) {
				System.out.println("ndbj.connectstring not defined in configuration file: + " + CONFIG_FILENAME);
			}

			NdbConfigurator.DB_NAME= System.getProperty("ndbj.testDatabase");
			if (NdbConfigurator.DB_NAME == null) {
				System.out.println("ndbj.testDatabase was empty. Setting it to 'test' db.");
			}

			NdbConfigurator.LIB_PATH= System.getProperty("ndbj.libpath");
			if (NdbConfigurator.LIB_PATH == null) {
				libPathSet=false;
			}
		}
		catch (java.io.IOException e) {
			throw new NdbApiException(e.getMessage());
			}
		catch (NullPointerException e) {
			throw new NdbApiException(e.getMessage());
			}
		catch (IllegalArgumentException e) {
			throw new NdbApiException(e.getMessage());
		}
		catch (SecurityException e) {
			throw new NdbApiException(e.getMessage());
		}

		if (libPathSet == true) {
			Class clazz = ClassLoader.class;
		    Field field;
		    boolean accessible;
		    try {
			    field = clazz.getDeclaredField("sys_paths");
			    accessible = field.isAccessible();
			    if (!accessible) {
			    	field.setAccessible(true);
			    }
			    field.set(clazz, null);
			    System.setProperty("java.library.path", LIB_PATH);
		    }
		    catch (NoSuchFieldException e) {
		        throw new NdbApiException(e.getMessage());
		    }
		    catch (IllegalAccessException e) {
		    	throw new NdbApiException(e.getMessage());
		    }
		}
	}
	private class SimpleAuthenticator
	   extends Authenticator
	{
	   private String username,
	                  password;

	   public SimpleAuthenticator(String username,String password)
	   {
	      this.username = username;
	      this.password = password;
	   }

	   protected PasswordAuthentication getPasswordAuthentication()
	   {
	      return new PasswordAuthentication(
	             username,password.toCharArray());
	   }

	}
}
